import React, { useCallback } from 'react';
import map from 'lodash/map';
import isEqual from 'lodash/isEqual';
import styles from './index.less';
import SortItem from './SortItem';
import { onLayoutSortChange } from 'lingxi-editor-core';
import ReactSortable from '../components/ReactSortable';
/**
 * 渲染排序节点
 * @param key
 * @param props
 */
function renderSortItems(key, props) {
    const { isFold, propName: parentPropName, specialProps: { domTreeKeys = [], key: parentKey } } = props;
    return (React.createElement(SortItem, { isFold: isFold, key: key, specialProps: { domTreeKeys: [...domTreeKeys, key], key, parentKey, parentPropName } }));
}
const putItem = (c, props) => {
    const { childNodes = [], nodeProps, componentName } = props;
    const fatherNodesRule = c.dataset.farules;
    const dragName = c.dataset.name;
    if (fatherNodesRule && !JSON.parse(fatherNodesRule).include(componentName))
        return false;
    if (nodeProps) {
        if (nodeProps.isOnlyNode && childNodes.length === 1)
            return false;
        if (nodeProps.childNodesRule) {
            return nodeProps.childNodesRule.includes(dragName);
        }
    }
    return true;
};
function SortTree(props) {
    const { childNodes, disabled, specialProps: { key }, propName } = props;
    /**
     * 拖拽排序
     * @param sortKeys
     * @param evt
     * @param props
     */
    const layoutSortChange = useCallback(function (sortKeys, evt) {
        if (isEqual(childNodes, sortKeys) || sortKeys.length < childNodes.length)
            return;
        /**
         * 获取拖住节点的信息
         * @type {any}
         */
        let dragInfo;
        if (sortKeys.length > childNodes.length) {
            dragInfo = JSON.parse(evt.clone.dataset.special);
        }
        onLayoutSortChange({
            sortKeys,
            parentKey: key,
            parentPropName: propName,
            dragInfo
        });
    }, [childNodes]);
    return (React.createElement(ReactSortable, { options: {
            group: { name: 'nested', put: (a, b, c) => putItem(c, props) },
            animation: 200,
            disabled,
            dataIdAttr: 'id',
            ghostClass: styles['item-background'],
            swapThreshold: 0.5,
        }, onChange: layoutSortChange }, map(childNodes, (key) => renderSortItems(key, props))));
}
export default SortTree;
