import React, { useEffect, useRef, useState } from 'react';
import { Tabs } from 'antd';
import map from 'lodash/map';
import filter from 'lodash/filter';
import get from 'lodash/get';
import isEmpty from 'lodash/isEmpty';
import isEqual from 'lodash/isEqual';
import cloneDeep from 'lodash/cloneDeep';
import SortComponent from './SortComponent';
import ObjectComponent from '../ObjectComponent/ObjectComponent';
import styles from '../../index.less';
import { formatPropsFieldConfigLocation } from '../../utils';
import { addPropsConfig } from 'lingxi-editor-core';
import { confirmModal } from '../index';
const { TabPane } = Tabs;
function ObjectArrayComponent(props, ref) {
    const { value, childPropsConfig, type, fatherFieldLocation, field, onChange } = props;
    let nextTabIndex = useRef(0), childPropsConfigRef = useRef(childPropsConfig);
    const [activeKey, setActiveKey] = useState('object0');
    const [panes, setPanes] = useState(map(value, (v, index) => ({
        value: v,
        childPropsConfig: get(childPropsConfig, `[${index}]`, {}),
        key: `object${nextTabIndex.current++}`,
    })));
    useEffect(() => {
        const resultData = [];
        for (let pane of panes) {
            if (!isEmpty(pane.value))
                resultData.push(pane.value);
        }
        if (isEmpty(panes)) {
            childPropsConfigRef.current = [];
        }
        else {
            childPropsConfigRef.current = map(panes, pane => pane.childPropsConfig);
        }
        let timer = setTimeout(() => {
            addPropsConfig({
                childPropsConfig: childPropsConfigRef.current,
                fatherFieldLocation: formatPropsFieldConfigLocation(type, field, fatherFieldLocation),
            });
            onChange && onChange(resultData);
        }, 500);
        return () => clearTimeout(timer);
    }, [panes, type, field, fatherFieldLocation]);
    useEffect(() => {
        if (!isEqual(childPropsConfig, childPropsConfigRef.current)) {
            childPropsConfigRef.current = childPropsConfig;
            setPanes(map(panes, (v, i) => {
                v.childPropsConfig = get(childPropsConfig, `[${i}]`, childPropsConfig[0]);
                return v;
            }));
        }
    }, [childPropsConfig]);
    function onEdit(targetKey, action) {
        const actionMap = {
            'add': add,
            'remove': remove,
        };
        actionMap[action](targetKey);
    }
    function add() {
        const key = `object${nextTabIndex.current++}`;
        setPanes([...panes, {
                key,
                childPropsConfig: cloneDeep(get(childPropsConfig, '[0]', {})),
                value: {},
            }]);
        setActiveKey(key);
    }
    function remove(targetKey) {
        confirmModal(() => removeTab(targetKey));
    }
    ;
    function removeTab(targetKey) {
        const nextPanes = filter(panes, pane => pane.key !== targetKey);
        if (activeKey === targetKey) {
            setActiveKey(get(nextPanes, '[0].key'));
        }
        if (nextPanes.length === 0) {
            nextTabIndex.current = 0;
        }
        setPanes(nextPanes);
    }
    function onDataChange(index, data) {
        const pane = panes[index];
        pane.value = data;
        setPanes([...panes]);
    }
    ;
    return (React.createElement("div", { className: styles['children-container'] },
        panes.length > 0 && (React.createElement(SortComponent, { onSortChange: (sortPanes) => setPanes(sortPanes), sortData: panes })),
        React.createElement(Tabs, { onChange: (changeActiveKey) => setActiveKey(changeActiveKey), activeKey: activeKey, type: "editable-card", style: { marginBottom: '12px' }, onEdit: onEdit }, panes.map((pane, index) => {
            const { key, value, childPropsConfig } = pane;
            const tabTitle = isEmpty(value) ? key : value[Object.keys(value)[0]];
            return (React.createElement(TabPane, { forceRender: true, tab: tabTitle, key: key },
                React.createElement(ObjectComponent, Object.assign({ isHideDivider: true }, props, { value: value, childPropsConfig: childPropsConfig, tabIndex: index, onChange: (data) => onDataChange(index, data) }))));
        }))));
}
export default ObjectArrayComponent;
