import React, { createElement, forwardRef, memo } from 'react';
import { Col, Form, Icon, Row, Tooltip } from 'antd';
import map from 'lodash/map';
import get from 'lodash/get';
import isEqual from 'lodash/isEqual';
import SelectAddProps from './SelectAddProps';
import styles from '../../index.less';
import { filterProps, formatPropsFieldConfigLocation } from '../../utils';
import isArray from 'lodash/isArray';
import each from 'lodash/each';
import { deletePropsConfig, PROPS_TYPES } from 'lingxi-editor-core';
import SwitchMultiTypes from '../SwitchMultiTypes';
import { confirmModal, TYPES_TO_COMPONENT } from '../../index';
const FormItem = Form.Item;
function ObjectComponent(props, ref) {
    const { fatherFieldLocation, type, tabIndex, form, childPropsConfig, field, isHideDivider } = props;
    function renderFormItemTitle(config, field) {
        const { label, tip, isAdd } = config;
        return (React.createElement("span", { className: styles['form-item-title-container'] },
            React.createElement(Tooltip, { title: tip || label }, label),
            isAdd && (React.createElement(Icon, { type: "delete", style: { paddingLeft: '7px' }, onClick: () => deletePropsConf(field) }))));
    }
    ;
    /**
     * 删除字段
     * @param field
     */
    function deletePropsConf(field) {
        confirmModal(() => deletePropsConfig({
            field,
            fatherFieldLocation: formatPropsFieldConfigLocation(type, props.field, fatherFieldLocation, tabIndex),
        }));
    }
    ;
    function renderFormItem(config, field) {
        const { getFieldDecorator } = form;
        const { type, rules } = config;
        const { defaultValue, value, ...restProps } = {
            field,
            ...config,
            fatherFieldLocation: formatPropsFieldConfigLocation(props.type, props.field, fatherFieldLocation, tabIndex),
            size: type === PROPS_TYPES.boolean ? 'default' : 'small',
        };
        if (isArray(type))
            return React.createElement(SwitchMultiTypes, { componentProps: restProps, key: `${field}`, extraObj: { rules }, field: field, types: type, form: form, tip: config.tip, label: config.label });
        return (React.createElement(Col, { span: 24, key: field },
            React.createElement(FormItem, { key: field, label: renderFormItemTitle(config, field) }, getFieldDecorator(field, { rules })(createElement(get(TYPES_TO_COMPONENT, type), restProps)))));
    }
    return (React.createElement(Form, { className: isHideDivider ? '' : styles.divider },
        React.createElement(Row, null, map(childPropsConfig, renderFormItem)),
        React.createElement(SelectAddProps, { fatherFieldLocation: formatPropsFieldConfigLocation(type, field, fatherFieldLocation, tabIndex) })));
}
export default Form.create({
    mapPropsToFields(props) {
        const { value } = props;
        const formatFields = {};
        each(value, (v, field) => (formatFields[field] = Form.createFormField({ value: v })));
        return formatFields;
    },
    onValuesChange: (props, _, allValues) => {
        const { onChange } = props;
        onChange && onChange(filterProps(allValues));
    },
})(memo(forwardRef(ObjectComponent), (prevProps, nextProps) => {
    const { childPropsConfig, tabIndex, value } = nextProps;
    const { childPropsConfig: prevChildPropsConfig, tabIndex: prevTabIndex, value: prevValue } = prevProps;
    return isEqual(childPropsConfig, prevChildPropsConfig) && isEqual(prevTabIndex, tabIndex) && isEqual(prevValue, value);
}));
